@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.CoordTrans
--- Copyright   :  (c) Sven Panne 2002-2006
+-- Copyright   :  (c) Sven Panne 2002-2009
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -20,7 +20,7 @@ module Graphics.Rendering.OpenGL.GL.CoordTrans (
 
    -- * Matrices
    MatrixMode(..), matrixMode,
-   Vector2(..), Vector3(..),
+   Vector1(..), Vector2(..), Vector3(..), Vector4(..),
    MatrixOrder(..), MatrixComponent(rotate,translate,scale), Matrix(..),
    currentMatrix, matrix, multMatrix, GLmatrix, loadIdentity,
    ortho, frustum, depthClamp,
@@ -51,8 +51,7 @@ import Graphics.Rendering.OpenGL.GL.BasicTypes (
 import Graphics.Rendering.OpenGL.GL.Exception ( bracket, unsafeBracket_ )
 import Graphics.Rendering.OpenGL.GL.Extensions (
    FunPtr, unsafePerformIO, Invoker, getProcAddress )
-import Graphics.Rendering.OpenGL.GL.PeekPoke (
-   peek1, peek2, peek3, peek4, poke2, poke3, poke4 )
+import Graphics.Rendering.OpenGL.GL.PeekPoke ( peek1, peek4, poke4 )
 import Graphics.Rendering.OpenGL.GL.QueryUtils (
    GetPName(GetDepthRange,GetViewport,GetMaxViewportDims,GetMatrixMode,
             GetModelviewMatrix,GetProjectionMatrix,
@@ -67,6 +66,8 @@ import Graphics.Rendering.OpenGL.GL.QueryUtils (
 import Graphics.Rendering.OpenGL.GL.StateVar (
    GettableStateVar, makeGettableStateVar, HasGetter(get), HasSetter(($=)),
    StateVar, makeStateVar )
+import Graphics.Rendering.OpenGL.GL.Tensor (
+   Vector1(..), Vector2(..), Vector3(..), Vector4(..) )
 import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit (
    marshalTextureUnit, unmarshalTextureUnit )
 import Graphics.Rendering.OpenGL.GL.VertexSpec ( TextureUnit )
@@ -209,28 +210,6 @@ foreign import CALLCONV unsafe "glMatrixMode" glMatrixMode :: GLenum -> IO ()
 
 --------------------------------------------------------------------------------
 
-data Vector2 a = Vector2 !a !a
-   deriving ( Eq, Ord, Show )
-
-instance Storable a => Storable (Vector2 a) where
-   sizeOf    ~(Vector2 x _) = 2 * sizeOf x
-   alignment ~(Vector2 x _) = alignment x
-   peek                     = peek2 Vector2 . castPtr
-   poke ptr   (Vector2 x y) = poke2 (castPtr ptr) x y
-
---------------------------------------------------------------------------------
-
-data Vector3 a = Vector3 !a !a !a
-   deriving ( Eq, Ord, Show )
-
-instance Storable a => Storable (Vector3 a) where
-   sizeOf    ~(Vector3 x _ _) = 3 * sizeOf x
-   alignment ~(Vector3 x _ _) = alignment x
-   peek                       = peek3 Vector3 . castPtr
-   poke ptr   (Vector3 x y z) = poke3 (castPtr ptr) x y z
-
---------------------------------------------------------------------------------
-
 data MatrixOrder = ColumnMajor | RowMajor
    deriving ( Eq, Ord, Show )
 
